// Demo 28: MousePadPanelB.java
//
//   A Panel to Act as a Mousepad in an Application such as 
//       AimPracticeB.
//   Illustrates Using the Mouse.
//   Also Draws an Aiming Circle. 
//
//           by Charles W. Neville
//    Copyright Charles W. Neville, July, 1999

// The user interface used in AimPracticeA.java for controlling 
// SimpleGunSight objects was rather clumsy.  A real video game
// would have to use a joystick or mouse to control the sight.
// MousePadPanelB.java and the next few demos illustrate how to do
// this.

// By the way, there is no MousePadPanelA class.  It seemed simpler
// to name this class "MousePadPanelB", because it goes with
// AimPracticeB.

// A MousePadPanelB object needs to know
//  1. Whom to report to (state variable).
//  2. How to set and return the object it reports to
//       (state variables).
//  3. How to convert its screen coordinates to the coordinates
//       of the object it reports to.
//  4. How to draw an Aiming Circle (override method paint())

// A MousePadPanelB object does not need to know
//  1. Where the mouse is pointing now, or where the mouse
//     was pointing when last clicked, moved or dragged.  This
//     is surprising, and the reason is contained below.

// We shall not actually process MouseEvents here.  Instead, we
// shall add the object we report to as a MouseListener, and
// let it process our MouseEvents.  This is probably the first
// time we have added an ActionListener or MouseListener to
// another object.  The effect is to DELEGATE to that object
// the task of handling our events.  That's why the Java 1.1
// event handling model is called "DELEGATION".  (By the way,
// the Java 1.0 even handling model, using handleEvent(), is
// called the "responder chain model".  Both the delegation 
// and responder chain models are general concepts of Object
// Oriented Programming, and neither is unique to Java (not
// even the names).

import java.awt.*;
import java.awt.event.*;  // for Listener interfaces

public class MousePadPanelB extends Panel {

   final int xOffset = 5;  // space at edges of Panel
   final int yOffset = 5;

   private Component reportTo;  // The Canvas, Panel or Frame we
                                // report MouseEvents to.

   // Constructor: initializes our reportTo Component, adds our
   //              our reportTo Component as a MouseListener,
   //              and sizes us.
   public MousePadPanelB(int xSize, int ySize, Component reportTo) {

      this.reportTo = reportTo;
      addMouseListener( (MouseListener) reportTo );
      addMouseMotionListener( (MouseMotionListener) reportTo );
      setSize(xSize, ySize);

   } // Constructor MousePadPanelB()

   // set our reportTo Component
   public void setReportTo(Component reportTo) {

      this.reportTo = reportTo;

   } // setReportTo

   // return our reportTo Component
   public Component getReportTo() {

      return reportTo;

   } // getReportTo()

   // simply be sure paint() is called promptly at
   // the beginning so the aiming circle is visible.
   public void init() {

      repaint();

   }  // init()

   // convert our screen x coordinate to the x coordinate
   // of the object we report to.
   public int convertX(int x) {
 
      return reportTo.getSize().width*x/this.getSize().width;

   } // convertX()

   // convert our screen y coordinate to the y coordinate
   // of the object we report to.
   public int convertY(int y) {

      return reportTo.getSize().height*y/this.getSize().height;

   } // convertY()
   

   public void paint(Graphics g) {

      int sizeX, sizeY;  // x and y components of our
                         // size, ie. our width and height.

      int d, r;    // diameter and radius of our outer
                   //   aiming circle
      int cx, cy;  // coordinates of aiming circles'
                   // centers

      sizeX = getSize().width;  sizeY = getSize().height;

      // draw a containing rectangle for appearance's sake.
      // leave some space at the edges of the panel
      g.drawRect(xOffset, yOffset, 
                 sizeX - 2*xOffset, sizeY - 2*yOffset);

      // find common center of aiming circles
      cx = sizeX/2; cy = sizeY/2;

      // make diameter the smaller of sizeX and sizeY
      d = sizeX;
      if(sizeY < d) d = sizeY;

      // make diameter 3/4 of the minimum of sizeX and sizeY
      d = 3*d/4;
      r = d/2;    // simple geometry!

      // draw aiming circles and direction lines
      g.drawOval(cx - r, cy - r, d, d);    // outer circle
      g.drawOval(cx - r/2, cy - r/2, d/2, d/2);
                                           // inner circle
      g.fillOval(cx - 2, cy - 2, 5, 5);    // exact center
      g.drawLine(xOffset, cy, 
                 sizeX - 2*xOffset, cy);   // horizontal
                                           // direction line
      g.drawLine(cx, yOffset, 
                 cx, sizeY - 2*yOffset);   // vertical
                                           // direction line

   } // paint()

} // MousePadPanelB 
